@import url(bootstrap-3.4.1-dist/css/bootstrap.min.css);
@import url(bootstrap-3.4.1-dist/css/bootstrap-datetimepicker.min.css);
/* Sticky footer styles
-------------------------------------------------- */
html {
  position: relative;
  min-height: 100%;
}
body {
  /* Margin bottom by footer height */
  margin-bottom: 60px;
}
.footer {
  position: absolute;
  bottom: 0;
  width: 100%;
  /* Set the fixed height of the footer here */
  height: 60px;
  line-height: 60px; /* Vertically center the text there */
  background-color: #f5f5f5;
}

.mr {
  margin-right: 5px;
}

.ml {
  margin-left: 5px;
}

/* Custom page CSS
-------------------------------------------------- */
/* Not required for template or sticky footer method. */

body > .container {
  padding: 60px 15px 0;
}

.footer > .container {
  padding-right: 15px;
  padding-left: 15px;
}
.footer { font-size: 10px; line-height: 30px; }
@media (min-width: 768px) {
.footer { font-size: 12px; line-height: 60px; }
}

code {
  font-size: 80%;
}

.navbar-header a.navbar-brand { padding-top: 0; padding-bottom: 0; }
@media (min-width: 768px) {
.navbar-fixed-top .container-fluid .navbar-collapse { font-size: 12px; }
.navbar-fixed-top .container-fluid .navbar-collapse li a { padding-left: 5px; padding-right: 5px; }
.navbar-fixed-top .container-fluid .navbar-right { font-size: 11px; }
.navbar-fixed-top .container-fluid .navbar-right .navbar-btn { padding: 5px; }
.navbar-header a.navbar-brand { padding: 0; }
}
@media (min-width: 992px) {
.navbar-header a.navbar-brand { padding: 0 15px; }
.navbar-fixed-top .container-fluid .navbar-collapse { font-size: 14px; }
.navbar-fixed-top .container-fluid .navbar-right { font-size: 12px; }
.navbar-fixed-top .container-fluid .navbar-collapse li a { padding-left: 10px; padding-right: 10px; }
.navbar-fixed-top .container-fluid .navbar-right { padding-right: 5px; }
.navbar-fixed-top .container-fluid .navbar-right .navbar-btn { margin-right: 5px; padding: 5px 10px; }
}
@media (min-width: 1200px) {
.navbar-fixed-top .container-fluid .navbar-right a { font-size: 14px; }
.navbar-fixed-top .container-fluid .navbar-right .navbar-btn { margin-right: 15px; }
.navbar-fixed-top .container-fluid .navbar-collapse li a { padding-left: 15px; padding-right: 15px; }
}


span.active { font-weight: bold; }
div#login{ max-width: 35em; }

.quota {
z-index:99;
height:14px;
position: absolute;
}
.quota_text {
  z-index:100;
  text-align: center;
  font-size: 12px;
  color: #666;
  cursor: default;
  position: absolute;
  width:120px;
  height:16px;
  margin-top:-17px;
  padding-top: -1px;
}
.quota_bg { background-color: white; z-index:98; width:122px; height:16px;margin-top:-1px;margin-left:-1px;   border: 1px solid #999;}
.quota_no_border { border:none; margin:0; }
.quota_high { background: url(../images/quota-colors.png) repeat-x 0 -28px #f90509; }
.quota_mid  { background: url(../images/quota-colors.png) repeat-x 0 -14px #e3e909; }
.quota_low  { background: url(../images/quota-colors.png) repeat-x 0 0px #05f905; }
.quota_text_high { color: white; }
.quota_text_mid { color: #666; }
.quota_text_low { color: #666; }

